#@ File (label="file of sequence") myfile
#@ String (choices={"P01","P02", "P03", "P04", "All", "noSubpositions"}, style="radioButtonHorizontal") location
#@ Integer (label = "nrRows", value = 8) nrRows
#@ Integer (label = "nrColumns", value = 12) nrColumns
#@ Integer (label = "nrChannels", value = 3) nrChannels
#@ String (choices={"Grays", "Blue", "Cyan", "Green", "Yellow", "Red", "Magenta"}, label = "Color Ch1", value= "Red") ch1
#@ String (choices={"Grays", "Blue", "Cyan", "Green", "Yellow", "Red", "Magenta"}, label = "Color Ch2", value = "Green") ch2
#@ String (choices={"Grays", "Blue", "Cyan", "Green", "Yellow", "Red", "Magenta"}, label = "Color Ch3", value = "Blue") ch3
#@ Integer (label = "border montage", value = 5) border
#@ Integer (label ="scale bar (um)") scalebar 
#@ Boolean (label="Annotations") annotation
#@ Boolean (label="Create single tiles") singletile
#@ Float (label= "Scaling montage", style="slider", min = 0.0, max = 1.0, value = 0.0, stepSize = 0.1) scale
#@ String (choices={"Jpeg", "png"}, label = "output RGB") outRGB
#@ String (choices={"hide", "show"}, label = "show/hide(faster) output") show

// This is somehow very memory hungry. You may need to restart ImageJ after a few rounds


setBatchMode(show);
if ("All" == location){
	points = newArray("P01", "P02", "P03", "P04");
} else if ("noSubpositions" == location) {
	points = newArray("");
} else {
	points = newArray(location);
}

for(ip = 0; ip < points.length; ip++){
	run("Close All");
	apoint = points[ip];
	outname = File.getNameWithoutExtension(myfile);
	outname = substring(outname, 0, lengthOf(outname)-7);
	outdirectory = File.getDirectory(File.getDirectory(myfile)) + "overview/";
	File.makeDirectory(outdirectory);
	stackfile = "/"+File.getDirectory(myfile) +  outname + apoint + "_stack.tif";
	outname = outdirectory + outname + apoint;
	outColor = newArray(3);
	outColor[0] = ch1;
	outColor[1] = ch2;
	outColor[2] = ch3;
	if(!File.exists(stackfile)){
		run("Image Sequence...", "open=" + myfile + " file=" + apoint + " sort");
		run("Stack to Hyperstack...", "order=xyczt(default) channels="+nrChannels+" slices=1 frames="+nrColumns*nrRows+" display=Composite");
		for(iCh = 0; iCh < nrChannels; iCh++){
			Stack.setChannel(iCh+1);
			run(outColor[iCh]);
		}
		saveAs("Tiff", stackfile);
		rename("stack");
	} else {
		open(stackfile);
		rename("stack");
		for(iCh = 0; iCh < nrChannels; iCh++){
			Stack.setChannel(iCh+1);
			run(outColor[iCh]);
		}
	}

	selectWindow("stack");
	minC = newArray(nrChannels);
	maxC = newArray(nrChannels);
	selectWindow("stack");
	run("Make Montage...", "columns="+nrColumns+" rows="+nrRows+" scale="+scale+" first=1 increment=1 border=0 use");
	for(iCh = 0; iCh < nrChannels; iCh++){
		offset = 3;
		Stack.setChannel(iCh+1);
		run("Enhance Contrast", "saturated=0.5");
		getMinAndMax(min, max);
		if (min+offset == max){
			minC[iCh] = min;
		} else {
			minC[iCh] = min+offset;
		}
		
		maxC[iCh] = max;
		setMinAndMax(minC[iCh], maxC[iCh]);
	}


	
	if (border > 0) {
		close("Montage");
		selectWindow("stack");
		setForegroundColor(0, 0, 0);
		run("Make Montage...", "columns="+nrColumns+" rows="+nrRows+" scale="+scale+" first=1 increment=1 border="+border+" use");
		for(iCh = 0; iCh < nrChannels; iCh++){
			Stack.setChannel(iCh+1);
			setMinAndMax(minC[iCh], maxC[iCh]);
		}	
	}
		

	


	saveAs("Tiff", outname + "_montage.tif");
	run("RGB Color");

	// Write positions of plate
	getDimensions(width, height, channels, slices, frames);
	lett = newArray("A", "B", "C", "D", "E", "F", "G", "H");
	num = newArray("1","2","3","4","5","6", "7", "8", "9","10","11","12");
	if (annotation){
		
		fontsize = maxOf(13, width/60);;
		setColor(255,255,255);  
		setFont("SansSerif", fontsize);
		
		for(i=0; i<nrRows; i++){
			ypos =  height/nrRows*(1/2 + i)+fontsize/2;
			setJustification("center");
			drawString(lett[i], fontsize/2, ypos);
		}
	
		for(i=0; i < nrColumns; i++){
			offset = (width-border*(nrColumns-1))/(2*nrColumns);
			xpos =  offset + (width/nrColumns)*i;
			setJustification("center");
			drawString(num[i], xpos, fontsize);
		}
	}
	
	if(scalebar>0){
		scalebarheight = maxOf(3, round(height/200));
		run("Scale Bar...", "width="+scalebar+" height="+scalebarheight +" font=28 color=White background=None location=[Lower Right] hide");
	}
	
	if (outRGB == "Jpeg")
		outnameRGB = outname + "_montage.jpg"; 
	if (outRGB == "png")
		outnameRGB = outname + "_montage.png";
	saveAs(outRGB, outnameRGB);
	
	// save single images for each panel
	if (singletile){
		selectWindow("stack");
		for(iCh = 0; iCh < nrChannels; iCh++){
			Stack.setChannel(iCh+1);
			setMinAndMax(minC[iCh], maxC[iCh]);
		}
		run("Flatten");
		lett = newArray("A", "B", "C", "D", "E", "F", "G", "H");
		num = newArray("01","02","03","04","05","06", "07", "08", "09","10","11","12");
		for(i=0; i < nrRows; i++){
			for(j=0; j < nrColumns ; j++){
				iframe = i*nrColumns+j+1;
				showProgress(iframe, nrColumns*nrRows);
				Stack.setFrame(iframe);
				outnamePos = outname + "_" + lett[i] + num[j] + ".png";
				saveAs("png", outnamePos);
			}
		}	
	}
}
